@@ -112,9 +112,9 @@ vec_proxy_order.default <- function(x, ...) {
 #' df <- data.frame(x = c(1, 1, 1, 2), y = c(0, 1, 2, 1))
 #' vec_compare(df, data.frame(x = 1, y = 1))
 vec_compare <- function(x, y, na_equal = FALSE, .ptype = NULL) {
-  vec_assert(x)
-  vec_assert(y)
-  vec_assert(na_equal, ptype = logical(), size = 1L)
+  obj_check_vector(x)
+  obj_check_vector(y)
+  check_bool(na_equal)
 
   args <- vec_recycle_common(x, y)
   args <- vec_cast_common_params(!!!args, .to = .ptype)
