@@ -15,13 +15,13 @@
 // -----------------------------------------------------------------------------
 
 // Pair with `PROTECT_GROUP_INFO()` in the caller
-struct group_info* new_group_info() {
+struct group_info* new_group_info(void) {
   SEXP self = PROTECT(r_new_raw(sizeof(struct group_info)));
   struct group_info* p_group_info = (struct group_info*) RAW(self);
 
   p_group_info->self = self;
   p_group_info->data_size = 0;
-  p_group_info->data = vctrs_shared_empty_int;
+  p_group_info->data = r_globals.empty_int;
   p_group_info->n_groups = 0;
   p_group_info->max_group_size = 0;
 
