@@ -1,3 +1,4 @@
+#include "vctrs-core.h"
 #include "vctrs.h"
 #include "type-data-frame.h"
 #include <R_ext/Rdynload.h>
@@ -62,11 +63,7 @@ static SEXP vctrs_eval_mask_n_impl(SEXP fn_sym, SEXP fn, SEXP* syms, SEXP* args,
  * @param env The environment in which to evaluate.
  */
 SEXP vctrs_eval_mask_n(SEXP fn, SEXP* syms, SEXP* args) {
-  SEXP mask = PROTECT(r_peek_frame());
-  SEXP out = vctrs_eval_mask_n_impl(R_NilValue, fn, syms, args, mask);
-
-  UNPROTECT(1);
-  return out;
+  return vctrs_eval_mask_n_impl(R_NilValue, fn, syms, args, vctrs_ns_env);
 }
 SEXP vctrs_eval_mask1(SEXP fn,
                       SEXP x_sym, SEXP x) {
@@ -208,7 +205,7 @@ SEXP vctrs_dispatch6(SEXP fn_sym, SEXP fn,
 }
 
 static SEXP vctrs_eval_mask_n_impl(SEXP fn_sym, SEXP fn, SEXP* syms, SEXP* args, SEXP env) {
-  SEXP mask = PROTECT(r_new_environment(env));
+  SEXP mask = PROTECT(r_alloc_empty_environment(env));
 
   if (fn_sym != R_NilValue) {
     Rf_defineVar(fn_sym, fn, mask);
@@ -239,7 +236,7 @@ SEXP vctrs_maybe_shared_col(SEXP x, SEXP i) {
 }
 
 // [[ register() ]]
-SEXP vctrs_new_df_unshared_col() {
+SEXP vctrs_new_df_unshared_col(void) {
   SEXP col = PROTECT(Rf_allocVector(INTSXP, 1));
   INTEGER(col)[0] = 1;
 
@@ -357,7 +354,10 @@ SEXP bare_df_map(SEXP df, SEXP (*fn)(SEXP)) {
   SEXP out = PROTECT(map(df, fn));
 
   // Total ownership because `map()` generates a fresh list
-  out = vec_bare_df_restore(out, df, vctrs_shared_zero_int, VCTRS_OWNED_true);
+  out = vec_bare_df_restore(out,
+                            df,
+                            VCTRS_OWNED_true,
+                            VCTRS_RECURSE_false);
 
   UNPROTECT(1);
   return out;
@@ -368,7 +368,10 @@ SEXP df_map(SEXP df, SEXP (*fn)(SEXP)) {
   SEXP out = PROTECT(map(df, fn));
 
   // Total ownership because `map()` generates a fresh list
-  out = vec_df_restore(out, df, vctrs_shared_zero_int, VCTRS_OWNED_true);
+  out = vec_df_restore(out,
+                       df,
+                       VCTRS_OWNED_true,
+                       VCTRS_RECURSE_false);
 
   UNPROTECT(1);
   return out;
@@ -479,6 +482,19 @@ SEXP vctrs_s3_find_method(SEXP generic, SEXP x, SEXP table) {
   return s3_find_method(r_chr_get_c_string(generic, 0), x, table);
 }
 
+// [[ register() ]]
+r_obj* ffi_s3_get_method(r_obj* generic, r_obj* class, r_obj* table) {
+  if (!r_is_string(generic)) {
+    r_stop_internal("`generic` must be a string");
+  }
+  if (!r_is_string(class)) {
+    r_stop_internal("`class` must be a string");
+  }
+  return s3_get_method(r_chr_get_c_string(generic, 0),
+                       r_chr_get_c_string(class, 0),
+                       table);
+}
+
 // [[ include("utils.h") ]]
 SEXP s3_find_method(const char* generic, SEXP x, SEXP table) {
   if (!OBJECT(x)) {
@@ -650,9 +666,9 @@ SEXP s4_class_find_method(SEXP class, SEXP table) {
 // [[ include("utils.h") ]]
 bool vec_implements_ptype2(SEXP x) {
   switch (vec_typeof(x)) {
-  case vctrs_type_scalar:
+  case VCTRS_TYPE_scalar:
     return false;
-  case vctrs_type_s3: {
+  case VCTRS_TYPE_s3: {
     SEXP method_sym = R_NilValue;
     SEXP method = s3_find_method_xy("vec_ptype2", x, x, vctrs_method_table, &method_sym);
 
@@ -784,16 +800,20 @@ bool list_has_inner_vec_names(SEXP x, R_len_t size) {
  * @return A list of the same length as `xs`.
  */
 // [[ include("utils.h") ]]
-SEXP list_pluck(SEXP xs, R_len_t i) {
-  R_len_t n = Rf_length(xs);
-  SEXP out = PROTECT(r_new_list(n));
+r_obj* list_pluck(r_obj* xs, r_ssize i) {
+  r_ssize n = r_length(xs);
+  r_obj* const * v_xs = r_list_cbegin(xs);
 
-  for (R_len_t j = 0; j < n; ++j) {
-    SEXP x = r_list_get(xs, j);
-    r_list_poke(out, j, r_list_get(x, i));
+  r_obj* out = KEEP(r_new_list(n));
+
+  for (r_ssize j = 0; j < n; ++j) {
+    r_obj* x = v_xs[j];
+    if (x != r_null) {
+      r_list_poke(out, j, r_list_get(x, i));
+    }
   }
 
-  UNPROTECT(1);
+  FREE(1);
   return out;
 }
 
@@ -802,7 +822,7 @@ SEXP list_pluck(SEXP xs, R_len_t i) {
 SEXP compact_seq_attrib = NULL;
 
 // p[0] = Start value
-// p[1] = Sequence size. Always >= 1.
+// p[1] = Sequence size. Always >= 0.
 // p[2] = Step size to increment/decrement `start` with
 void init_compact_seq(int* p, R_len_t start, R_len_t size, bool increasing) {
   int step = increasing ? 1 : -1;
@@ -936,6 +956,15 @@ SEXP colnames(SEXP x) {
                          syms_x, x);
 }
 
+r_obj* colnames2(r_obj* x) {
+  r_obj* names = colnames(x);
+  if (names == r_null) {
+    return r_alloc_character(Rf_ncols(x));
+  } else {
+    return names;
+  }
+}
+
 // [[ include("utils.h") ]]
 bool is_integer64(SEXP x) {
   return TYPEOF(x) == REALSXP && Rf_inherits(x, "integer64");
@@ -1150,23 +1179,6 @@ static SEXP new_env_call = NULL;
 static SEXP new_env__parent_node = NULL;
 static SEXP new_env__size_node = NULL;
 
-#if 0
-SEXP r_new_environment(SEXP parent, R_len_t size) {
-  parent = parent ? parent : R_EmptyEnv;
-  SETCAR(new_env__parent_node, parent);
-
-  size = size ? size : 29;
-  SETCAR(new_env__size_node, Rf_ScalarInteger(size));
-
-  SEXP env = Rf_eval(new_env_call, R_BaseEnv);
-
-  // Free for gc
-  SETCAR(new_env__parent_node, R_NilValue);
-
-  return env;
-}
-#endif
-
 // [[ include("utils.h") ]]
 SEXP r_protect(SEXP x) {
   return Rf_lang2(fns_quote, x);
@@ -1468,16 +1480,7 @@ bool vctrs_debug_verbose = false;
 SEXP vctrs_ns_env = NULL;
 SEXP vctrs_shared_empty_str = NULL;
 
-SEXP vctrs_shared_empty_lgl = NULL;
-SEXP vctrs_shared_empty_int = NULL;
-SEXP vctrs_shared_empty_dbl = NULL;
-SEXP vctrs_shared_empty_cpl = NULL;
-SEXP vctrs_shared_empty_chr = NULL;
-SEXP vctrs_shared_empty_raw = NULL;
-SEXP vctrs_shared_empty_list = NULL;
 SEXP vctrs_shared_empty_date = NULL;
-SEXP vctrs_shared_true = NULL;
-SEXP vctrs_shared_false = NULL;
 
 Rcomplex vctrs_shared_na_cpl;
 SEXP vctrs_shared_na_lgl = NULL;
@@ -1493,6 +1496,8 @@ SEXP strings_minimal = NULL;
 SEXP strings_unique = NULL;
 SEXP strings_universal = NULL;
 SEXP strings_check_unique = NULL;
+SEXP strings_unique_quiet = NULL;
+SEXP strings_universal_quiet = NULL;
 SEXP strings_key = NULL;
 SEXP strings_loc = NULL;
 SEXP strings_val = NULL;
@@ -1529,8 +1534,10 @@ SEXP chrs_asc = NULL;
 SEXP chrs_desc = NULL;
 SEXP chrs_largest = NULL;
 SEXP chrs_smallest = NULL;
+SEXP chrs_which = NULL;
 
 SEXP syms_i = NULL;
+SEXP syms_j = NULL;
 SEXP syms_n = NULL;
 SEXP syms_x = NULL;
 SEXP syms_y = NULL;
@@ -1539,7 +1546,6 @@ SEXP syms_y_size = NULL;
 SEXP syms_to = NULL;
 SEXP syms_dots = NULL;
 SEXP syms_bracket = NULL;
-SEXP syms_arg = NULL;
 SEXP syms_x_arg = NULL;
 SEXP syms_y_arg = NULL;
 SEXP syms_to_arg = NULL;
@@ -1583,6 +1589,10 @@ SEXP syms_stop_matches_remaining = NULL;
 SEXP syms_stop_matches_incomplete = NULL;
 SEXP syms_stop_matches_multiple = NULL;
 SEXP syms_warn_matches_multiple = NULL;
+SEXP syms_stop_matches_relationship_one_to_one = NULL;
+SEXP syms_stop_matches_relationship_one_to_many = NULL;
+SEXP syms_stop_matches_relationship_many_to_one = NULL;
+SEXP syms_warn_matches_relationship_many_to_many = NULL;
 SEXP syms_action = NULL;
 SEXP syms_vctrs_common_class_fallback = NULL;
 SEXP syms_fallback_class = NULL;
@@ -1592,6 +1602,8 @@ SEXP syms_chr_proxy_collate = NULL;
 SEXP syms_actual = NULL;
 SEXP syms_required = NULL;
 SEXP syms_call = NULL;
+SEXP syms_dot_call = NULL;
+SEXP syms_which = NULL;
 
 SEXP fns_bracket = NULL;
 SEXP fns_quote = NULL;
@@ -1613,7 +1625,7 @@ SEXP r_new_shared_character(const char* name) {
   return out;
 }
 
-void c_print_backtrace() {
+void c_print_backtrace(void) {
 #if defined(RLIB_DEBUG)
 #include <execinfo.h>
 #include <stdlib.h>
@@ -1632,8 +1644,6 @@ void c_print_backtrace() {
 }
 
 
-struct r_lazy r_lazy_null;
-
 void vctrs_init_utils(SEXP ns) {
   vctrs_ns_env = ns;
 
@@ -1651,7 +1661,7 @@ void vctrs_init_utils(SEXP ns) {
 
   // Holds the CHARSXP objects because unlike symbols they can be
   // garbage collected
-  strings2 = r_new_shared_vector(STRSXP, 23);
+  strings2 = r_new_shared_vector(STRSXP, 25);
 
   strings_dots = Rf_mkChar("...");
   SET_STRING_ELT(strings2, 0, strings_dots);
@@ -1686,6 +1696,12 @@ void vctrs_init_utils(SEXP ns) {
   strings_check_unique = Rf_mkChar("check_unique");
   SET_STRING_ELT(strings2, 10, strings_check_unique);
 
+  strings_unique_quiet = Rf_mkChar("unique_quiet");
+  SET_STRING_ELT(strings2, 23, strings_unique_quiet);
+
+  strings_universal_quiet = Rf_mkChar("universal_quiet");
+  SET_STRING_ELT(strings2, 24, strings_universal_quiet);
+
   strings_key = Rf_mkChar("key");
   SET_STRING_ELT(strings2, 11, strings_key);
 
@@ -1767,6 +1783,7 @@ void vctrs_init_utils(SEXP ns) {
   chrs_desc = r_new_shared_character("desc");
   chrs_largest = r_new_shared_character("largest");
   chrs_smallest = r_new_shared_character("smallest");
+  chrs_which = r_new_shared_character("which");
 
   classes_tibble = r_new_shared_vector(STRSXP, 3);
 
@@ -1784,22 +1801,9 @@ void vctrs_init_utils(SEXP ns) {
   SET_STRING_ELT(classes_vctrs_group_rle, 2, Rf_mkChar("vctrs_vctr"));
 
 
-  vctrs_shared_empty_lgl = r_new_shared_vector(LGLSXP, 0);
-  vctrs_shared_empty_int = r_new_shared_vector(INTSXP, 0);
-  vctrs_shared_empty_dbl = r_new_shared_vector(REALSXP, 0);
-  vctrs_shared_empty_cpl = r_new_shared_vector(CPLXSXP, 0);
-  vctrs_shared_empty_chr = r_new_shared_vector(STRSXP, 0);
-  vctrs_shared_empty_raw = r_new_shared_vector(RAWSXP, 0);
-  vctrs_shared_empty_list = r_new_shared_vector(VECSXP, 0);
   vctrs_shared_empty_date = r_new_shared_vector(REALSXP, 0);
   Rf_setAttrib(vctrs_shared_empty_date, R_ClassSymbol, classes_date);
 
-  vctrs_shared_true = r_new_shared_vector(LGLSXP, 1);
-  LOGICAL(vctrs_shared_true)[0] = 1;
-
-  vctrs_shared_false = r_new_shared_vector(LGLSXP, 1);
-  LOGICAL(vctrs_shared_false)[0] = 0;
-
   vctrs_shared_na_cpl.i = NA_REAL;
   vctrs_shared_na_cpl.r = NA_REAL;
 
@@ -1813,6 +1817,7 @@ void vctrs_init_utils(SEXP ns) {
   INTEGER(vctrs_shared_zero_int)[0] = 0;
 
   syms_i = Rf_install("i");
+  syms_j = Rf_install("j");
   syms_n = Rf_install("n");
   syms_x = Rf_install("x");
   syms_y = Rf_install("y");
@@ -1821,7 +1826,6 @@ void vctrs_init_utils(SEXP ns) {
   syms_to = Rf_install("to");
   syms_dots = Rf_install("...");
   syms_bracket = Rf_install("[");
-  syms_arg = Rf_install("arg");
   syms_x_arg = Rf_install("x_arg");
   syms_y_arg = Rf_install("y_arg");
   syms_to_arg = Rf_install("to_arg");
@@ -1857,7 +1861,6 @@ void vctrs_init_utils(SEXP ns) {
   syms_parent = Rf_install("parent");
   syms_s3_methods_table = Rf_install(".__S3MethodsTable__.");
   syms_from_dispatch = Rf_install("vctrs:::from_dispatch");
-  syms_df_fallback = Rf_install("vctrs:::df_fallback");
   syms_s3_fallback = Rf_install("vctrs:::s3_fallback");
   syms_stop_incompatible_type = Rf_install("stop_incompatible_type");
   syms_stop_incompatible_size = Rf_install("stop_incompatible_size");
@@ -1867,6 +1870,10 @@ void vctrs_init_utils(SEXP ns) {
   syms_stop_matches_incomplete = Rf_install("stop_matches_incomplete");
   syms_stop_matches_multiple = Rf_install("stop_matches_multiple");
   syms_warn_matches_multiple = Rf_install("warn_matches_multiple");
+  syms_stop_matches_relationship_one_to_one = Rf_install("stop_matches_relationship_one_to_one");
+  syms_stop_matches_relationship_one_to_many = Rf_install("stop_matches_relationship_one_to_many");
+  syms_stop_matches_relationship_many_to_one = Rf_install("stop_matches_relationship_many_to_one");
+  syms_warn_matches_relationship_many_to_many = Rf_install("warn_matches_relationship_many_to_many");
   syms_action = Rf_install("action");
   syms_vctrs_common_class_fallback = Rf_install(c_strs_vctrs_common_class_fallback);
   syms_fallback_class = Rf_install("fallback_class");
@@ -1876,6 +1883,8 @@ void vctrs_init_utils(SEXP ns) {
   syms_actual = Rf_install("actual");
   syms_required = Rf_install("required");
   syms_call = Rf_install("call");
+  syms_dot_call = Rf_install(".call");
+  syms_which = Rf_install("which");
 
   fns_bracket = Rf_findVar(syms_bracket, R_BaseEnv);
   fns_quote = Rf_findVar(Rf_install("quote"), R_BaseEnv);
@@ -1926,8 +1935,6 @@ void vctrs_init_utils(SEXP ns) {
     UNPROTECT(4);
   }
 
-  r_lazy_null = (struct r_lazy) { 0 };
-
   // We assume the following in `union vctrs_dbl_indicator`
   VCTRS_ASSERT(sizeof(double) == sizeof(int64_t));
   VCTRS_ASSERT(sizeof(double) == 2 * sizeof(int));
